@@ -6,7 +6,7 @@ import signal
 from typing import Set
 import warnings
 import sys
-from bluetti_mqtt.bluetooth import BluetoothClientHandler, scan_devices
+from bluetti_mqtt.bluetooth import BluetoothClientHandler, check_addresses, scan_devices
 from bluetti_mqtt.bus import EventBus
 from bluetti_mqtt.mqtt_client import MQTTClient
 
@@ -51,6 +51,12 @@ class CommandLineHandler:
             metavar='ADDRESS',
             nargs='*',
             help='The device MAC(s) to connect to')
+
+        # The default event loop on windows doesn't support add_reader, which
+        # is required by asyncio-mqtt
+        if sys.platform == 'win32':
+            asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+
         args = parser.parse_args()
         if args.scan:
             asyncio.run(scan_devices())
@@ -65,12 +71,12 @@ class CommandLineHandler:
 
         # Verify that we can see all the given addresses
         addresses = set(args.addresses)
-        bluetooth_handler = BluetoothClientHandler(addresses, args.interval, bus)
-        devices = await bluetooth_handler.check()
+        devices = await check_addresses(addresses)
         if len(devices) == 0:
             sys.exit('Could not find the given devices to connect to')
 
         # Start bluetooth handler (manages connections)
+        bluetooth_handler = BluetoothClientHandler(devices, args.interval, bus)
         self.bluetooth_task = loop.create_task(bluetooth_handler.run())
 
         # Start MQTT client
@@ -85,9 +91,10 @@ class CommandLineHandler:
         self.mqtt_task = loop.create_task(mqtt_client.run())
 
         # Register signal handlers for safe shutdown
-        signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
-        for s in signals:
-            loop.add_signal_handler(s, lambda: asyncio.create_task(self.shutdown()))
+        if sys.platform != 'win32':
+            signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
+            for s in signals:
+                loop.add_signal_handler(s, lambda: asyncio.create_task(self.shutdown()))
 
         # Run until cancelled
         try:
@@ -114,3 +121,6 @@ def main(argv=None):
 
     cli = CommandLineHandler(argv)
     cli.execute()
+
+if __name__ == "__main__":
+    main(sys.argv)
\ No newline at end of file
